<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Software License Agreement
 * that is bundled with this package in the file LICENSE.txt.
 *
 * @author    SecurePay - https://www.securepay.com.au/
 * @copyright PrestaShop SA
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class SecureApi extends PaymentModule
{
    private $_html = '';
    private $_postErrors = array();

    public $address;

    protected $backward = false;
    public $test_gateway_url = 'https://payment.securepay.com.au/test/v2/invoice';
    public $live_gateway_url = 'https://payment.securepay.com.au/v2/invoice';

    /**
     * SecureFrame constructor.
     *
     * Set the information about this module
     */
    public function __construct()
    {
        $this->name                   = 'secureapi';
        $this->tab                    = 'payments_gateways';
        $this->version                = '1.4;
        $this->author                 = 'SecureAPI';
        $this->controllers            = array('payment', 'validation', 'success');
        $this->currencies             = true;
        $this->currencies_mode        = 'checkbox';
        $this->bootstrap              = true;
        $this->displayName            = 'SecurePay API';
        $this->description            = 'Accept online credit card payments via SecurePay API - Australia Post';
        $this->confirmUninstall       = 'Are you sure about removing these details??';
        $this->ps_versions_compliancy = array('min' => '1.7.0', 'max' => _PS_VERSION_);
	$this->module_key = '86395056f1d8307a18b581e985420f0d';

        $config = Configuration::getMultiple(
            array(
                'SECURE_API_CLIENT_ID',
                'SECURE_API_CLIENT_SECRET',
                'SECURE_API_MERCHANT_CODE',
                'SECURE_API_STORE_ID',
                'SECURE_API_TEST_MODE',
                'SECURE_API_TRANSACTION_TYPE',
                'SECURE_API_SAVE_CARD',
                'SECURE_API_REFUND_STATUS')
        );
        $this->client_id = $config['SECURE_API_CLIENT_ID'];
        $this->client_secret = $config['SECURE_API_CLIENT_SECRET'];
        $this->merchant_code = $config['SECURE_API_MERCHANT_CODE'];
        $this->store_id = $config['SECURE_API_STORE_ID'];
        $this->test_mode = $config['SECURE_API_TEST_MODE'];
        $this->transaction_type = $config['SECURE_API_TRANSACTION_TYPE'];
        $this->refund_status_id = $config['SECURE_API_REFUND_STATUS'];
        $this->save_card = $config['SECURE_API_SAVE_CARD'];

        $this->bootstrap = true;

        parent::__construct();
    }

    /**
     * Install this module and register the following Hooks:
     *
     * @return bool
     */
    public function install()
    {
        return parent::install()
            && $this->registerHook('payment')
            && $this->registerHook('adminOrder')
            && $this->registerHook('paymentOptions')
            && $this->registerHook('paymentReturn')
            && $this->registerHook('displayHeader')
            && $this->registerHook('displayBackOfficeOrderActions')
            && $this->installDB();
    }

    /**
     * Uninstall this module and remove it from all hooks
     *
     * @return bool
     */
    public function uninstall()
    {
        if (!Configuration::deleteByName('SECURE_API_MERCHANT_ID') || !Configuration::deleteByName('SECURE_API_CLIENT_SECRET') || !Configuration::deleteByName('SECURE_API_TEST_MODE') || !Configuration::deleteByName('SECURE_API_CLIENT_SECRET') || !Configuration::deleteByName('SECURE_API_TRANSACTION_TYPE') || !Configuration::deleteByName('SECURE_API_REFUND_STATUS') || !Configuration::deleteByName('SECURE_API_SAVE_CARD') || ! $this->uninstallDb() || !parent::uninstall()) {
            return false;
        }
        return parent::uninstall();
    }

    /**
     * Returns a string containing the HTML necessary to
     * generate a configuration screen on the admin
     *
     * @return string
     */
    public function getContent()
    {
        if (Tools::isSubmit('btnSubmit')) {
            $this->postValidation();
            if (!count($this->_postErrors)) {
                $this->postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        } else {
            $this->_html .= '<br />';
        }
        $this->_html .= $this->displaySecureFrame();
        $this->_html .= $this->renderForm();
        return $this->_html;
    }

    /**
     * Display this module as a payment option during the checkout
     *
     * @param array $params
     * @return array|void
     */
    public function hookPaymentOptions($params)
    {
        /*
         * Verify if this module is active
         */
        if (!$this->active) {
            return;
        }
        /**
         * Form action URL. The form data will be sent to the
         * validation controller when the user finishes
         * the order process.
         */
        $formAction = $this->context->link->getModuleLink($this->name, 'payment', array(), true);
        /**
         * Assign the url form action to the template var $action
         */
        $this->smarty->assign(['action' => $formAction]);

        $externalOption = new PaymentOption;
        $externalOption
            ->setCallToActionText($this->l($this->displayName))
            ->setAction($this->context->link->getModuleLink($this->name, 'payment', array(), true))
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/logo.png'))
            ->setModuleName($this->name);


        $payment_options = array(
            $externalOption
        );
        return $payment_options;
    }

    public function installDb()
    {
        $return = true;
        $sql = $this->getSqlTable();
        foreach ($sql as $s) {
            $return &= Db::getInstance()->execute($s);
        }

        return $return;
    }

    public function uninstallDb()
    {
        $sql = $this->getSqlTable();
        foreach ($sql as $name => $v) {
            Db::getInstance()->execute('DROP TABLE '.$name);
        }

        return true;
    }
    /**
     * Display a message in the paymentReturn hook
     * @param array $params
     * @return string
     */
    public function hookPaymentReturn($parameters)
    {
        /**
         * Verify if this module is enabled
         */
        if (!$this->active) {
            return;
        }

//        $state = $parameters['order']->getCurrentState();

        $currencyOrder = new Currency($parameters['order']->id_currency);

//        $template = '';

//        if ($state == Configuration::get('PS_OS_PAYMENT') || $state == Configuration::get('PS_OS_OUTOFSTOCK') || $state == Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')) {
        $this->smarty->assign(array(
            'shop_name' => $this->context->shop->name,
            'total_to_pay' => Tools::displayPrice($parameters['order']->total_paid, $currencyOrder),
            'status' => 'ok',
            'id_order' => $parameters['order']->id
        ));
        if (isset($parameters['order']->reference) && !empty($parameters['order']->reference)) {
            $this->smarty->assign('reference', $parameters['order']->reference);
        }
//        }


        return $this->fetch('module:secureapi/views/templates/hook/payment_return.tpl');
    }

    private function postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('SECURE_API_CLIENT_ID')) {
                $this->_postErrors[] = $this->l('Client ID required.');
            } elseif (!Tools::getValue('SECURE_API_CLIENT_SECRET')) {
                $this->_postErrors[] = $this->l('Client Recret is required.');
            } elseif (!Tools::getValue('SECURE_API_MERCHANT_CODE')) {
                $this->_postErrors[] = $this->l('MerchantCode is required.');
            } elseif (!Tools::getValue('SECURE_API_STORE_ID')) {
                $this->_postErrors[] = $this->l('store id is required.');
            }
        }
    }

    private function displaySecureFrame()
    {
        return $this->display(__FILE__, 'infos.tpl');
    }


    public function renderForm()
    {
        $fields_form = array();
        $fields_form[0] = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('SecureAPI Account Details'),
                    'icon' => 'icon-envelope'
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('Client ID (*)'),
                        'name' => 'SECURE_API_CLIENT_ID',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Client Secret (*)'),
                        'name' => 'SECURE_API_CLIENT_SECRET',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('MerchantCode (*)'),
                        'name' => 'SECURE_API_MERCHANT_CODE',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Store Id (*)'),
                        'name' => 'SECURE_API_STORE_ID',
                    ),
                    array(
                        //'type' => 'radio',
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Test Mode'),
                        'name' => 'SECURE_API_TEST_MODE',
                        'values' => array(
                            array(
                                'value' => 1,
                                'label' => $this->l('Yes')
                            ),
                            array(
                                'value' => 0,
                                'label' => $this->l('No')
                            ),
                        )
                    ),


                    array(
                        'type' => 'select',
                        'label' => $this->l('Transaction Type'),
                        'name' => 'SECURE_API_TRANSACTION_TYPE',
                        'options' => array(
                            'query' => self::getTypeOptions(),
                            'id' => 'id', 'name' => 'name'
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Refund Order status'),
                        'name' => 'SECURE_API_REFUND_STATUS',
                        'options' => array(
                            'query' => OrderState::getOrderStates($this->context->language->id),
                            'id' => 'id_order_state',
                            'name' => 'name'
                        )
                    ),
                    array(
                        //'type' => 'radio',
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Save card'),
                        'name' => 'SECURE_API_SAVE_CARD',
                        'values' => array(
                            array(
                                'value' => 1,
                                'label' => $this->l('Yes')
                            ),
                            array(
                                'value' => 0,
                                'label' => $this->l('No')
                            ),
                        )
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );



        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
        $helper->allow_employee_form_lang = $form_lang ? $form_lang : 0;
        $this->fields_form = array();
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $params = '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . $params;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );
        return $helper->generateForm($fields_form);
    }

    protected static function getTypeOptions()
    {
        $data = array(
            array('id' => 'PAYMENT', 'name' => 'PAYMENT'),
            array('id' => 'PREAUTH', 'name' => 'PREAUTH'),
        );
        return $data;
    }

    public function getConfigFieldsValues()
    {
        $client_id = Tools::getValue('SECURE_API_CLIENT_ID', Configuration::get('SECURE_API_CLIENT_ID'));
        $client_secret = Configuration::get('SECURE_API_CLIENT_SECRET');
        $merchant_code = Configuration::get('SECURE_API_MERCHANT_CODE');
        $store_id = Configuration::get('SECURE_API_STORE_ID');
        $mod = Tools::getValue('SECURE_API_TEST_MODE', Configuration::get('SECURE_API_TEST_MODE'));

        $con_tran_type = Configuration::get('SECURE_API_TRANSACTION_TYPE');
        $tran_type = Tools::getValue('SECURE_API_TRANSACTION_TYPE', $con_tran_type);
        $status = Tools::getValue('SECURE_API_REFUND_STATUS', Configuration::get('SECURE_API_REFUND_STATUS'));
        $save_card = Tools::getValue('SECURE_API_SAVE_CARD', Configuration::get('SECURE_API_SAVE_CARD'));

        return array(
            'SECURE_API_CLIENT_ID' => $client_id,
            'SECURE_API_CLIENT_SECRET' => $client_secret,
            'SECURE_API_MERCHANT_CODE' => $merchant_code,
            'SECURE_API_STORE_ID' => $store_id,
            'SECURE_API_TEST_MODE' => $mod,
            'SECURE_API_TRANSACTION_TYPE' => $tran_type,
            'SECURE_API_REFUND_STATUS' => $status,
            'SECURE_API_SAVE_CARD' => $save_card,

        );
    }

    private function postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            $client_id = Tools::getValue('SECURE_API_CLIENT_ID');
            Configuration::updateValue('SECURE_API_CLIENT_ID', $client_id);
            $client_recret = Tools::getValue('SECURE_API_CLIENT_SECRET');
            Configuration::updateValue('SECURE_API_CLIENT_SECRET', $client_recret);

            $merchant_code = Tools::getValue('SECURE_API_MERCHANT_CODE');
            Configuration::updateValue('SECURE_API_MERCHANT_CODE', $merchant_code);
            $store_id = Tools::getValue('SECURE_API_STORE_ID');
            Configuration::updateValue('SECURE_API_STORE_ID', $store_id);
            Configuration::updateValue('SECURE_API_TEST_MODE', Tools::getValue('SECURE_API_TEST_MODE'));
            $tran_type = Tools::getValue('SECURE_API_TRANSACTION_TYPE');
            Configuration::updateValue('SECURE_API_TRANSACTION_TYPE', $tran_type);
            Configuration::updateValue('SECURE_API_REFUND_STATUS', Tools::getValue('SECURE_API_REFUND_STATUS'));
            Configuration::updateValue('SECURE_API_SAVE_CARD', Tools::getValue('SECURE_API_SAVE_CARD'));
        }
        $this->_html .= $this->displayConfirmation($this->l('Settings updated'));
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function hookDisplayHeader()
    {
        $this->context->controller->registerStylesheet('modules-secureapi-css', 'modules/'.$this->name.'/views/css/styles.css', ['media' => 'all', 'priority' => 150]);
    }

    public function hookDisplayBackOfficeOrderActions($params)
    {
        $order_id = (int)$params['id_order'];
        $order = new Order($order_id);
        $module = $order->module;
        $link_capture_payment = $this->context->link->getAdminLink('AdminSecureapi', true).'&order_id='.$order_id;
        $link_cancel_preauth_payment = $this->context->link->getAdminLink('AdminSecureapiCancel', true).'&order_id='.$order_id;
        //check order preauth or payment
        if ($module=='secureapi') {
            $secure_payment = Db::getInstance()->getRow(
                'SELECT secure_order_type FROM ' . _DB_PREFIX_ . 'secure_api_order_status  WHERE cart_id = ' .$order_id
            );
            $secure_order_type = $secure_payment['secure_order_type'];
            $this->context->smarty->assign(
                array(
                'order' => $order,
                'link_capture_payment'=>$link_capture_payment,
                'link_cancel_preauth_payment'=>$link_cancel_preauth_payment,
                'secure_order_type'=>$secure_order_type
                )
            );
            return $this->display(__FILE__, 'admin.tpl');
        }
    }

    private function getSqlTable()
    {
        $sql = array();
        $sql[_DB_PREFIX_.'secure_api_saved_card'] = 'CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'secure_api_saved_card` (
			  `id` int(20) NOT NULL AUTO_INCREMENT,
			  `user_id` int(20) NOT NULL,
			  `card_token` varchar(64),
			  `customer_code` varchar(50),
			  `brand_type` varchar(50),
			  `brand_category` varchar(50),
			  `scheme` varchar(50),
			  `bin` varchar(50),
			  `last4` varchar(50),
			  `expiry_month` varchar(50),
			  `expiry_year` varchar(50),
			  `created_time` varchar(255),
			  PRIMARY KEY (`id`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';

        $sql[_DB_PREFIX_.'secure_api_order_status'] = 'CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'secure_api_order_status` (
			  `id` int(20) NOT NULL AUTO_INCREMENT,
			  `cart_id` int(20) NOT NULL,
			  `order_id` varchar(20),
			  `secure_order_type` varchar(50),
			  `created_time` datetime NOT NULL,
			  `updated_time` datetime NOT NULL,
			  PRIMARY KEY (`id`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';
        return $sql;
    }
}
